import Globals, AccessControl, Acquisition
import os
import sys
import string
from OFS import Folder, SimpleItem
from Products.Formulator.Form import ZMIForm
from Products.PythonScripts.PythonScript import PythonScript

from Globals import DTMLFile, Persistent
from Products.ZSQLMethods.SQL import *
from OFS.SimpleItem import Item

class GFolder(
  Folder.Folder
  ):

  "GFolder by Gregory Vincic"

  meta_type = 'GFolder'

  """Set which tabs to show for this GFolder object
  """

  manage_options = (
    Folder.Folder.manage_options[0:2] +
    (
    {'label':'Generic', 'action':'formGeneric'},
    ) +
    Folder.Folder.manage_options[2:7]
    )

  
  def __init__(self, id, title):
    """Initialize our GFolder
    """
    self.id = id
    self.title = title


  manage_Generic = DTMLFile('dtml/formGeneric', globals())

  # Added by Gregory Vincic

  def formGeneric(self, REQUEST, connection_id=None, table_id=None):
    """Generic settings
    """
    types_map = {'int':'IntegerField',
                 'float':'FloatField',
                 'string':'StringField',
                 'date':'DateTimeField',
                 'timestamp':'DateTimeField',    # Ignored for now
                 'varchar':'TextAreaField',
                 'text':'TextAreaField',
                 'nb':'StringField',
                 }

    if connection_id != None:
      # Get information about the tables in this database
      info = self._infoFactory(connection_id)
      
      # Generate fields for the selected table_id
      if table_id != None:
        fields_to_generate = info.getColumnsNames([table_id])
      
        # Find out where we are
        dir = string.split(globals()['__file__'], '/')
        dir.pop()
        dir = string.join(dir, '/')                

        # Create the form object
        my_form = ZMIForm('form', 'My form')

        # Find out what type of field this is and create it
        message = []
        for column in fields_to_generate[table_id]:
          type = info.type(table_id, column)
          # Only add fields that are not already there
          if my_form.has_field(column):
            tuple = column , "Already exists"
            message.append(tuple)
          else:
            # Timestamp fields we ignore since they are supposed to be updated by the database when changes occur
            if type != 'timestamp':
              my_form.manage_addField(column, column, types_map[type], REQUEST)
              tuple = column, type
              message.append(tuple)

        # Add the created form
        self._setObject('form', my_form)

        # Create sql_insert statement

        args = []
        # We remove all timestamp fields
        for x in fields_to_generate[table_id]:
          if info.type(table_id, x) != 'timestamp':
            args.append(x)


        # We asume that mos tables have an auto increment columnt
        primary_id = args.pop(0)

        values = []
        for x in args:
          values.append("<dtml-sqlvar %s type=string>\n\t" % (x))

        command = "insert into %s (%s) values(%s)" % (
            table_id,
            string.join(args, ","),
            string.join(values, ",")
            )

        self._setObject('sql_insert', SQL('sql_insert', '',
                                                    connection_id,
                                                    string.join(args, "\n"),
                                                    command))

        # Create sql_update statement
        values = []

        for x in args:
          values.append("%s = <dtml-sqlvar %s type=string>\n\t" % (x, x))

        command = "update %s set %s where %s = <dtml-sqlvar %s type=string>" % (
            table_id,                   
            string.join(values, ","),
            primary_id,
            primary_id
            )

        # We need to have the primary_id as an argument to the sql_update statement
        args.append(primary_id)

        self._setObject('sql_update', SQL('sql_update', '',
                                                    connection_id,
                                                    string.join(args, "\n"),
                                                    command))

        # Create sql_delete statement

        command = "delete from %s where %s = <dtml-sqlvar %s type=string>" % (
            table_id,                   
            primary_id,
            primary_id
            )

        self._setObject('sql_delete', SQL('sql_delete', '',
                                                    connection_id,
                                                    primary_id,
                                                    command))

        # Create sql_list statement

        command = "select %s from %s " % (
            string.join(args, ","),
            table_id
            )

        self._setObject('sql_list', SQL('sql_list', '',
                                                    connection_id,
                                                    '',
                                                    command))

        # Create sql_get statement

        command = "select %s from %s where %s = <dtml-sqlvar %s type=string>" % (
            string.join(args, ","),
            table_id,                   
            primary_id,
            primary_id
            )

        self._setObject('sql_get', SQL('sql_get', '',
                                                    connection_id,
                                                    primary_id,
                                                    command))


        # Create the view document
        view_file = dir + '/dtml/view_form.dtml'
        doc = string.join(open(view_file).readlines(), '') % ( primary_id,
                                                           'sql_get',
                                                           primary_id,
                                                           primary_id,
                                                           primary_id,
                                                           primary_id
                                                           )

        self.manage_addDTMLDocument('view_form', '', doc)

        # Create the index_html document

        values = []
        for x in args:
          values.append("<dtml-var %s>," % (x))

        index_file = dir + '/dtml/index_html.dtml'
        doc = string.join(open(index_file).readlines()) % ( primary_id,
                                                            primary_id,
                                                            primary_id
                                                            )

        self.manage_addDTMLDocument('index_html', '', doc)

        return self.view_generic_form(self, REQUEST, info=info, manage_tabs_message=message)

      return self.view_generic_form(self, REQUEST, info=info)

    return self.view_generic_form(self, REQUEST)




  def _infoFactory(self, connection_id):
    connection = getattr(self, connection_id)
    exec('import DBInfo.' + connection.database_type)
    return eval('DBInfo.' + connection.database_type + '.Info(connection)')



  view_generic_form = DTMLFile('dtml/formGeneric', globals())

  


manage_addGFolderForm = DTMLFile('dtml/folderAdd', globals())

def manage_addGFolder(self, id, title, RESPONSE):
  """Add GFolder object
  """
  self._setObject(id, GFolder(id, title))
  return RESPONSE.redirect(id + '/manage_main')

  



